--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TotemManager.  If not, see <http://www.gnu.org/licenses/>.
--]]


Factory = {};

-- returns type, singleton
function Factory.CreateSingletonType(superType, ...)
  local newType = {};
  local singleton;
  if superType~=nil then
    setmetatable(newType, {__index=superType});
    local factorySingleton = {};
    factorySingleton.instanceType = newType;
    local superFactoryType = superType.factory;
    setmetatable(factorySingleton, {__index = superFactoryType});
    factorySingleton.currentFactoryType = superFactoryType;
    singleton = superFactoryType.Create(factorySingleton, ...); -- we call the super create
  else
    singleton = {};
    setmetatable(singleton, {__index=newType});
  end
  return newType, singleton;
end

-- returns type, factoryType
function Factory.CreateAbstractType(superType)
  local newType = {};
  local factoryType = {};
  if superType==nil then
    setmetatable(factoryType, {__index=Factory});
  else
    setmetatable(newType, {__index=superType});
    setmetatable(factoryType, {__index=superType.factory});
    factoryType.superType = superType.factory;
  end
  factoryType.Create = Factory.Create;
  newType.factory = factoryType;
  return newType, factoryType;
end

-- returns type, factoryType, factorySingleton
function Factory.CreateType(superType)
  local newType = {};
  local factoryType = {};
  if superType==nil then
    setmetatable(factoryType, {__index=Factory});  
  else
    setmetatable(newType, {__index=superType});
    setmetatable(factoryType, {__index=superType.factory});
    factoryType.superType = superType.factory;
  end
  factoryType.Create = Factory.Create;
  newType.factory = factoryType;
  local factorySingleton = {};
  factorySingleton.instanceType = newType;
  factorySingleton.factoryType = factoryType;
  factorySingleton.currentFactoryType = factoryType;
  setmetatable(factorySingleton, {__index = factoryType, __newindex=function (_, n) 
    error("A factory singleton should not change its variables ("..n..")");
  end});
  return newType, factoryType, factorySingleton;
end

-- this method should be overrided
function Factory:Create(...)
  return self:NewInstance(...);
end

function Factory:NewInstance(...)
  local superFactoryType = self.currentFactoryType.superType;
  if superFactoryType == nil then
    -- we can create our instance
    local instance = {};
    setmetatable(instance, {__index=self.instanceType});
    -- and reset currentFactoryType
    rawset(self, "currentFactoryType", self.factoryType);
    return instance;  
  else
    rawset(self, "currentFactoryType", superFactoryType);
    return superFactoryType.Create(self, ...); -- we call the super create
  end
end

function Factory:Cast(instance)
  setmetatable(instance, {__index=self.instanceType});
end

setmetatable(Factory, {__newindex=function () 
  error("Factory should not be modified");
end});
