--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TotemManager.  If not, see <http://www.gnu.org/licenses/>.
--]]


--[[
  The CandyBarTimerRenderer class.
--]]

-- CandyBarTimerRenderer extends TimerRenderer
CandyBarTimerRenderer, CandyBarTimerRendererFactory, candyBarTimerRendererFactory = Factory.CreateType(TimerRenderer);

CandyBarTimerRenderer.LEFT_DIRECTION = 1;
CandyBarTimerRenderer.RIGHT_DIRECTION = 2;

-- ignore name (1st arg)
function CandyBarTimerRenderer:StartTimer(_, duration, texture, target, count, tickFrequency, tickStart)
  self.target = target;
  self:SetCount(count)
  self.candyBar:SetCandyBarIcon(self.name, texture);
  self.candyBar:SetTime(self.name, duration);
  self.candyBar:Start(self.name);
end

function CandyBarTimerRenderer:StopTimer()
  self.candyBar:Stop(self.name);
end

function CandyBarTimerRenderer:SetCount(count)
  if count ~= -1 then
    self.candyBar:SetText(self.name, count..". "..self.target);
  else
    self.candyBar:SetText(self.name, self.target);
  end
end

function CandyBarTimerRenderer:SetDirection(direction)
  if direction==self.LEFT_DIRECTION then
    -- self.candyBar:SetCandyBarReversed(self.name, false);
  elseif direction==self.RIGHT_DIRECTION then
    -- self.candyBar:SetCandyBarReversed(self.name, true);
  end
end

function CandyBarTimerRenderer:SetPosition(parent, left, top)
  self.candyBar:SetCandyBarPoint(self.name, "LEFT", parent, "TOPLEFT", left, -top);
end

--[[
  The CandyBarTimerRendererFactory class.
--]]

-- scale did not work
function CandyBarTimerRendererFactory:Create(name, scale)
  candyBarTimerRenderer = self:NewInstance();

  candyBarTimerRenderer.name = name;
  CandyBarTimerRenderer.candyBar:Register(name, 1, "", nil, "green", "red");
  candyBarTimerRenderer:SetPosition(UIParent, 0, 0);
  -- self.candyBar:SetCandyBarScale(name, scale);
  return candyBarTimerRenderer;
end

function CandyBarTimerRendererFactory:VARIABLES_LOADED()
  CandyBarTimerRenderer.candyBar = {};
  setmetatable(CandyBarTimerRenderer.candyBar, {__index = LibStub("CandyBar-2.0")});
  environment:UnregisterEvent("VARIABLES_LOADED", self);
end

environment:RegisterEvent("VARIABLES_LOADED", candyBarTimerRendererFactory);


