--[[
    TotemManager
    Copyright (C) 2008 Gael Lalire
    Contact: gael.lalire@laposte.net

    TotemManager is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TotemManager is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TotemManager.  If not, see <http://www.gnu.org/licenses/>.
--]]

--[[
  The CircleActionBarRenderer class.
--]]

-- CircleActionBarRenderer extends ActionBarRenderer
CircleActionBarRenderer, CircleActionBarRendererFactory, circleActionBarRendererFactory = Factory.CreateType(ActionBarRenderer);

-- this method will be deprecated
function CircleActionBarRenderer:CreateSecureUnitButton(parent, position)
  local secureUnitButton = CreateFrame("Button", nil, parent, "SecureUnitButtonTemplate");
  secureUnitButton.borderTexture = secureUnitButton:CreateTexture(nil, "OVERLAY");
  
  secureUnitButton.targetText = secureUnitButton:CreateFontString(nil, "ARTWORK", "NumberFontNormal");
  secureUnitButton.targetText:SetAllPoints();
  
  secureUnitButton.cooldown = CreateFrame("Cooldown", nil, secureUnitButton, "CooldownFrameTemplate");  
  
  secureUnitButton.renderer = self;
  secureUnitButton.position = position;
  
  secureUnitButton:SetScript("OnEnter", function (button)
    if button.renderer:IsTooltipAsked() then
      button.renderer:UpdateTooltip(button);      
    end
  end);

  secureUnitButton:SetScript("OnLeave", function ()
    GameTooltip:Hide();
  end);
  
  self:ApplySkin(secureUnitButton, position);
  return secureUnitButton;
end

function CircleActionBarRenderer:SPELL_UPDATE_COOLDOWN()
  local secureUnitButton = self.selectedSecureUnitButton;
  local action = secureUnitButton.currentAction;
  if action ~= nil then
    local start, duration = action:GetCooldown();
    CooldownFrame_SetTimer(secureUnitButton.cooldown, start, duration, 1);
  end
  for i=1, #self.secureUnitButton do
    secureUnitButton = self.secureUnitButton[i];  
    action = secureUnitButton.currentAction;
    if action ~= nil then
      start, duration = action:GetCooldown();
      CooldownFrame_SetTimer(secureUnitButton.cooldown, start, duration, 1);
    end
  end
end

function CircleActionBarRenderer:ApplySkin(secureUnitButton, position)
  -- Show all graphical adds to the button
  local buttonSize = self.buttonSide;
  secureUnitButton:SetWidth(buttonSize);
  secureUnitButton:SetHeight(buttonSize);
  secureUnitButton:SetPoint("CENTER", self.parent, "TOPLEFT", self.left+position*(self.buttonDiameter*math.cos(self.angle)), -(self.top-position*(self.buttonDiameter*math.sin(self.angle))));

  local borderSize = self.buttonSide*.3;
  local borderTexture = secureUnitButton.borderTexture;
  borderTexture:SetPoint("TOPLEFT", secureUnitButton, -borderSize, borderSize);
  borderTexture:SetPoint("BOTTOMRIGHT", secureUnitButton, borderSize, -borderSize);
  if position == 0 then
    self.selectedSecureUnitButton = secureUnitButton; -- todo : add a lot of information here
    borderTexture:SetTexture("Interface\\AddOns\\TotemManager\\gui\\textures\\SerenityBaseGreen");  
  else
    self.secureUnitButton[position] = secureUnitButton; -- todo : add a lot of information here ... { ... , borderTexture, targetText, ... }
    borderTexture:SetTexture("Interface\\AddOns\\TotemManager\\gui\\textures\\SerenityBase");  
  end
end

function CircleActionBarRenderer:UpdateTooltip(secureUnitButton)
  GameTooltip:SetOwner(secureUnitButton, "ANCHOR_RIGHT");
  secureUnitButton.currentAction:ShowInTooltip(GameTooltip);
  if self.tooltipHelpAdded then
    GameTooltip:AddLine(" ");
    if secureUnitButton.position == 0 then
      self:AddSelectedTooltipHelp(GameTooltip);
    else
      if self.selectedSecureUnitButton:IsVisible() then
        self:AddFavoriteTooltipHelp(GameTooltip);
      else
        self:AddOpenTooltipHelp(GameTooltip);
      end
    end
  end
  GameTooltip:Show();
end

function CircleActionBarRenderer:UnApplySkin(secureUnitButton, position)
  -- hide all graphical adds to the button
  -- but keep all created font, frame, cooldown ...
  -- detach them if possible
end


function CircleActionBarRenderer:ActionChanged(secureUnitButton, action)
  secureUnitButton.currentAction = action;
  
  if action ~= nil then
    local start, duration = action:GetCooldown();
    CooldownFrame_SetTimer(secureUnitButton.cooldown, start, duration, 1);
  
    if GameTooltip:GetOwner()==secureUnitButton then
      self:UpdateTooltip(secureUnitButton);
    end
    local texture = action:GetTexture();
    secureUnitButton:SetNormalTexture(texture);
    
    local target = action:GetTarget();
    if target=="target" then
      secureUnitButton.targetText:SetText("");
    elseif target=="player" then
      secureUnitButton.targetText:SetText("P");    
    elseif target=="focus" then
      secureUnitButton.targetText:SetText("F");
    elseif target=="targettarget" then
      secureUnitButton.targetText:SetText("A");  
    else
      secureUnitButton.targetText:SetText("?");     
    end
  end
end

function CircleActionBarRenderer:UpdateAllSkin()
  self:ApplySkin(self.selectedSecureUnitButton, 0);
  for i, secureUnitButton in ipairs(self.secureUnitButton) do
    self:ApplySkin(secureUnitButton, i);
  end
end

function CircleActionBarRenderer:SetAngle(angle)
  self.angle = angle;
  self:UpdateAllSkin();
end

function CircleActionBarRenderer:SetPosition(parent, left, top)
  self.parent = parent;
  self.left = left;
  self.top = top;
  self:UpdateAllSkin();  
end

--[[
  The CircleActionBarRendererFactory class.
--]]

-- left and top indicate the position of the first button's center
function CircleActionBarRendererFactory:Create(buttonDiameter)
  local circleActionBarRenderer = self:NewInstance();
  
  environment:RegisterEvent("SPELL_UPDATE_COOLDOWN", circleActionBarRenderer);
  
  circleActionBarRenderer.secureUnitButton = {};
  circleActionBarRenderer.parent = UIParent;
  circleActionBarRenderer.left = 0;
  circleActionBarRenderer.top = 0;
  circleActionBarRenderer.angle = 0;
  circleActionBarRenderer.buttonDiameter = buttonDiameter;
  circleActionBarRenderer.buttonSide = buttonDiameter / 1.4142; -- sqrt(2)
  return circleActionBarRenderer;
end
